(function($){
	
	/**
	 * Elements properties editor
	 * @class ElementEditor
	 */
	$.fg.ElementEditor = {
			
			/**
			 * Element editor area DOM node
			 * @type {object}
			 */
			object: null,
			/**
			 * Id of current editing element
			 * @type {number}
			 */
			currentElementId: null,
		
		/**
		 * Initialize the Element Editor
		 * @return {void}
		 */
		init: function()
		{
			this.object = $( "#element-editor" );
			this.object.resizable( {
				handles: "n",
				containment: "#work-panel",
				minHeight: this.object.height()
			} );
			this.object.find( "a.button-close" ).click( this.closeHandler );
			$(this.object).surfacing();
		},
		
		/**
		 * Get current editing element object from storage
		 * @return {$.fg.Element} element
		 */
		getCurrentElement: function()
		{
			return $.fg.Elements.getFromStorage(this.currentElementId);
		},
		
		/**
		 * Show editor area
		 * @return {void}
		 */
		show: function()
		{
			var container = this.getCurrentElement().containerElement;
			if(container) {
				var row = container.getRow( this.currentElementId );
				if(row) {
					row.addClass( "action-edit" );
				}
			}
			if(this.object.is( ":hidden" )) {
				//this.object.slideDown( 200 );
				this.object.show( "slide", {direction: "down"}, 200 );
			}
		},
		
		/**
		 * Hide editor area
		 * @return {void}
		 */
		hide: function()
		{
			if(this.getCurrentElement()) {
				var container = this.getCurrentElement().containerElement;
				if(container) {
					var row = container.getRow( this.currentElementId );
					if(row) {
						row.removeClass( "action-edit" );
					}
				}
				this.currentElementId = null;
			}
			if(this.object.is( ":visible" )) {
				this.object.hide( "slide", {direction: "down"}, 200 );
				//this.object.slideUp( 200 );
			}
		},
		
		/**
		 * Set title of the editor area
		 * @param {string} title
		 * @return {void}
		 */
		setTitle: function(title)
		{
			this.object.find( "h4" ).text(title);
		},
		
		/**
		 * Clear editor area content
		 * @return {void}
		 */
		clearContent: function()
		{
			this.getContent().text('');
		},
		
		/**
		 * Set editor area content
		 * @param {mixed} content string or DOM node
		 * @return {void}
		 */
		setContent: function(content)
		{
			this.clearContent();
			this.getContent().append(content);
		},
		
		/**
		 * Get editor area content DOM node
		 * @return {object} content DOM node
		 */
		getContent: function()
		{
			return this.object.find( ".content" );
		},
		
		/**
		 * Set editor class attribute
		 * @param {string} classValue css name
		 * @return {void}
		 */
		setEditorClass: function(classValue)
		{
			this.object.attr( "class", classValue );
		},
		
		/**
		 * Fire edit of the element
		 * @param {number} elementId Id of the element to edit
		 * @return {void}
		 */
		edit: function(elementId)
		{
			var element = $.fg.Elements.getFromStorage(elementId);
			if(this.currentElementId) {
				var container = this.getCurrentElement().containerElement;
				if(container) {
					var row = container.getRow( this.currentElementId );
					if(row) {
						row.removeClass( "action-edit" );
					}
				}
			}
			this.currentElementId = elementId;
			this.setTitle( $.String.sub(
				t( "element-editor.editor-title" ),
				{element: t( "elements."+element.getProperty( "type" )+".title" )}
			) );
			this.setEditorClass( "element-" + element.type );
			
			// default ui with element properties
			if(element.properties)
			{
				var table = $( "<table>" ).addClass( "properties" );
				$.each(element.properties, function( k, v ) {
					var renderedProperty = element.renderProperty( k );
					if(renderedProperty) {
						table.append(
							$( "<tr>" )
								.addClass( "property-"+k )
								.append(
									$( "<td>" )
										.addClass( "key" )
										.append(
											$( "<span>" )
												.addClass( "property-name" )
												.text( v.displayName || k )
										)
								)
								.append(
									$( "<td></td>" )
										.addClass( "value" )
										.append( element.renderProperty( k ) )
								)
						);
					}
				});
				this.setContent(table);
				this.object.find( ".content .properties :input" )
					.change( this.propertyChangeHandler )
					.focus(function() {
						this.oldValue = this.value;
					})
					.change(function() {
						if(this.oldValue != this.value) {
							var elementId = $.fg.ElementEditor.currentElementId;
							var propertyName = $(this).attr( "name" );
							var oldValue = this.oldValue;
							var value = this.value;
							undoRedo(
								function() {
									var element = $.fg.Elements.getFromStorage( elementId );
									element.setProperty( propertyName, oldValue );
									$.fg.ElementEditor.edit( elementId );
									$.fg.ElementEditor.updateElement( elementId );
								},
								function() {
									var element = $.fg.Elements.getFromStorage( elementId );
									element.setProperty( propertyName, value );
									$.fg.ElementEditor.edit( element.id );
									$.fg.ElementEditor.updateElement( element.id );
								}
							);
						}
					});
			}
			else
			{
				this.setContent( t( "element-editor.element-has-no-properties" ) );
			}
			
			// maby element will draw its own editor ui
			element.edit( this );
			
			this.show();
		},
		
		/**
		 * Rerender current editing element row
		 * @return {void}
		 */
		updateElement: function()
		{
			var element = this.getCurrentElement();
			if(element.containerElement) {
				element.containerElement.updateRow( element.id );
			}
		},
		
		/**
		 * Handler of element property change (form inputs)
		 * @param {event} e
		 */
		propertyChangeHandler: function(e)
		{
			var propertyInput = $( e.target );
			$.fg.ElementEditor.getCurrentElement().setProperty( propertyInput.attr( "name" ), propertyInput.val() );
			$.fg.ElementEditor.updateElement();
		},
		
		/**
		 * Handler of click on close button
		 * @param {event} e
		 */
		closeHandler: function(e)
		{
			e.preventDefault();
			$.fg.ElementEditor.hide();
		}
		
	};
	
})(jQuery);