<?php

/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * This is the model class for table "{{order}}".
 *
 * The followings are the available columns in table '{{order}}':
 * @property string $id
 * @property integer $ordered
 * @property string $user_id
 * @property string $shipping_info
 * @property string $payment_info
 */
class Order extends CActiveRecord
{
    private $_userId;

    const ORDERED = 1;
    const PROCESS = 0;

    public $verifyCode;

	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return Order the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
	
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{order}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('ordered', 'numerical', 'integerOnly'=>true),
            array('shipping_info, payment_info', 'length', 'max'=>1024),
			array('user_id', 'length', 'max'=>10),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, ordered, user_id, tmp_user_id, shipping_info, payment_info', 'safe', 'on'=>'search'),
            array('verifyCode','captcha','allowEmpty'=>!Yii::app()->user->isGuest || !CCaptcha::checkRequirements() || !Yii::app()->getModule('shoppingcart')->captchaEnabled, 'captchaAction' => '/shoppingcart/shoppingcart/captcha'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
            'order_items'    => array(self::HAS_MANY,   'Item',    'order_id'),
		);
	}
    /**
     * @return array named scopes.
     */
    public function scopes()
    {
        return array(
            'ordered'=>array(
                'condition'=>'ordered='.self::ORDERED,
            ),
            'process'=>array(
                'condition'=>'ordered='.self::PROCESS,
            ),
        );
    }

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id'      => ShoppingcartModule::t('ID'),
			'ordered' => ShoppingcartModule::t('Ordered'),
			'user_id' => ShoppingcartModule::t('User'),
            'shipping_info' => ShoppingcartModule::t('Shipping Info'),
            'payment_info' => ShoppingcartModule::t('Payment Info'),
            'verifyCode' => ShoppingcartModule::t('Verify Code'),
		);
	}
    /*
     * The method returns the model of actual order or its id
     * @param integer $id_only  bool value that determines what will be returned
     * @return object | integer
     */
    public static function getOrder($id_only = false){

        $user_id = self::getUser(true);
        if($user_id === false){
            return false;
        }
        $cacheId = self::getOrderCacheId($user_id);
        if (!($model = Yii::app()->cache->get($cacheId, false))) {
            $model = self::model()->process()->find('user_id=:user_id',array(':user_id' =>$user_id));
            if (!isset($model)) {
                $model = new Order;
                $model->user_id  = $user_id;
                $model->save(false);
            }
            Yii::app()->cache->set($cacheId, $model, Yii::app()->getModule('shoppingcart')->cacheTime);
        }

        return $id_only? $model->id: $model;
    }
    /*
     * The method returns the model od id of user who is owner of actual order
     * @param integer $id_only  bool value that determines what will be returned
     * @return object | integer
     */
    public static function getUser($id_only = false)
    {
        $user_id = Yii::app()->user->getState('user_id', null);
        $cacheId = self::getUserCacheId($user_id);
        if (!($user = Yii::app()->cache->get($cacheId, false))) {
            $user = null;
            if (!isset($user_id) && isset(Yii::app()->user->id)) {
                $user = TmpUser::model()->with('order')->find('t.parent_user_id=:parent_user_id AND order.ordered=:process',
                    array(':parent_user_id'=>Yii::app()->user->id,':process'=>Order::PROCESS));
            }
            if (!isset($user_id) && !isset($user)) {
                $user_id = (string)Yii::app()->request->cookies['user_id'];
            }
            if (isset($user_id) && !isset($user)) {
                $user = TmpUser::model()->findByPk($user_id);
            }
            if (!isset($user)) {
                if(self::getItemIsAdded()){
                    $user = new TmpUser;
                    if(isset(Yii::app()->user->id)){
                        $user->parent_user_id = Yii::app()->user->id;
                    }
                    $user->save(false);
                    Yii::app()->user->setState('user_id', $user->id);
                    if (!headers_sent()) {
                        $cookie = new CHttpCookie('user_id', $user->id);
                        $cookie->expire = time()+60 * Yii::app()->getModule('shoppingcart')->max_lifetime;
                        $cookie->domain=Yii::app()->getModule('shoppingcart')->cookie_domain;
                        Yii::app()->request->cookies['user_id'] = $cookie;
                    }
                }else{
                    return false;
                }
            }
            if (isset(Yii::app()->user->id) && !isset($user->parent_user_id)) {
                $user->parent_user_id = Yii::app()->user->id;
                $user->save(false, array('parent_user_id'));
            }

            $cacheId = self::getUserCacheId($user->id);
            Yii::app()->cache->set($cacheId, $user, Yii::app()->getModule('shoppingcart')->cacheTime);
        }

        return $id_only? $user->id: $user;
    }
    /*
     * The method checks if item is added right now
     * @return boolean if item is added
     */
    public static function getItemIsAdded(){
        return (Yii::app()->controller->id === 'item'
            && Yii::app()->controller->action->id === 'add')
            || Yii::app()->controller->id === 'phpunit';
    }
    /*
     * The method generates cache id for current user
     * @param integer $user_id  current user id
     * @return string cache id
     */
    public static function getUserCacheId($user_id){
        $cache_id = $user_id;
        if (isset(Yii::app()->user->id)) {
            $cache_id .= Yii::app()->user->id;
        }
        return "customer_" . md5($cache_id);
    }
    /*
     * The method generates cache id for order
     * @param integer $user_id  current user id
     * @return string cache id
     */
    public static function getOrderCacheId($user_id){
        return 'order_' . md5($user_id);
    }
    /*
     * The method generates cache id for order items
     * @param integer $user_id  current user id
     * @return string cache id
     */
    public static function getOrderItemsCacheId($user_id){
        return 'order_items_' . md5($user_id);
    }
    /*
     * The method clears order cache.
     * It is used when something is changing in the order
     */
    public static function flushOrder() {
        $user_id = self::getUser(true);
        $idsToFlush = array(
            self::getOrderCacheId($user_id),
            self::getOrderItemsCacheId($user_id)
        );
        foreach ($idsToFlush as $cacheId) {
            Yii::app()->cache->delete($cacheId);
        }
    }
    /*
     * The method clears user's cache.
     * It is used when something is changing in the user's data
     */
    public static function flushUser()
    {
        $user_id = Yii::app()->user->getState('user_id', false);
        if ($user_id) {
            $cacheId = self::getUserCacheId($user_id);
            Yii::app()->cache->delete($cacheId);
        }
    }
	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('id',$this->id,true);
		$criteria->compare('ordered',$this->ordered);
		$criteria->compare('user_id',$this->user_id,true);
        $criteria->compare('shipping_info',$this->shipping_info,true);
        $criteria->compare('payment_info',$this->order_info,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

    /**
     * The function that deletes unnecessary items, orders and users
     * lifetime of which has already gone
     *
     * @return void
     */
    static function cleaning(){

        $max_lifetime = Yii::app()->getModule('shoppingcart')->max_lifetime;

        $connection = Yii::app()->db;
        //Find all process orders which should be removed

        Yii::beginProfile('garbageCollector');

        $ordersToRemove = $command=$connection
            ->createCommand(
            "SELECT ProcessOrder.id AS order_id,
                    ProcessOrder.user_id,
                    IF((DATE_ADD(IFNULL(MAX(Item.ordered_at),ProcessOrder.started_at), INTERVAL :max_lifetime MINUTE) < NOW()) , TRUE , FALSE) AS order_to_remove,
                    IF((COUNT(OrderedOrder.id) < 1) , TRUE, FALSE) AS user_to_remove
             FROM {{order}} AS ProcessOrder
             LEFT JOIN {{item}} AS Item
                    ON ProcessOrder.id = Item.order_id
             LEFT JOIN {{order}} AS OrderedOrder
                    ON ProcessOrder.user_id = OrderedOrder.user_id AND OrderedOrder.ordered = :ordered
             WHERE ProcessOrder.ordered=:process GROUP BY ProcessOrder.id
             HAVING order_to_remove = TRUE")
            ->bindValues(array(
                ':max_lifetime' => $max_lifetime,
                ':ordered'      => Order::ORDERED,
                ':process'      => Order::PROCESS
            ))->queryAll();

        $orderIds = array();
        $userIds  = array();
        foreach($ordersToRemove as $order){
            $orderIds[] = $order['order_id'];
            if($order['user_to_remove']){
                $userIds[] = $order['user_id'];
            }
        }
        $orderIds   = array_unique($orderIds);
        $userIds    = array_unique($userIds);
        //Delete items
        if(count($orderIds)){
            $criteria=new CDbCriteria;
            $criteria->addInCondition('order_id', $orderIds);
            Item::model()->deleteAll($criteria);
        }
        //Delete orders
        if(count($orderIds)){
            $criteria=new CDbCriteria;
            $criteria->addInCondition('id', $orderIds);
            Order::model()->deleteAll($criteria);
        }
        //Delete users
        if(count($orderIds)){
            $criteria=new CDbCriteria;
            $criteria->addInCondition('id', $userIds);
            TmpUser::model()->deleteAll($criteria);
        }
        Yii::endProfile('garbageCollector');
    }
    /**
     * The function performs caching of the order items
     *
     * @return array order items
     */
    public function getItems(){

        $user_id = self::getUser(true);
        $cacheId = self::getOrderItemsCacheId($user_id);
        if (!($data = Yii::app()->cache->get($cacheId, false))) {
            $data = $this->order_items;
            Yii::app()->cache->set($cacheId, $data, Yii::app()->getModule('shoppingcart')->cacheTime);
        }
        return $data;
    }

}
